"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from constants import AMC_PATH, ENTERPRISE_FLAG_FILE
import sys, datetime, os.path
sys.path.insert(1,'%s/site-packages/' % AMC_PATH)

from flask import Flask
from libs.flask_compress import Compress
from utilities import background_tasks, config_manager

app = Flask(__name__)
app.config["UPDATE_INTERVAL"] = int(config_manager.get_update_interval())
app.config["COMPRESS_DEBUG"] = True
app.config["COMPRESS_MIMETYPES"] = ["text/javascript", "application/javascript", "application/x-javascript", "image/png", "text/html", "text/css"]

Compress(app)

from flask import json, request, render_template
from controllers import cluster_controller, namespace_controller
from controllers import node_controller, udf_controller
from controllers import crud_controller
from utilities.cluster_manager import ClusterManager
import eventlet

cluster_manager = ClusterManager()


# URLs for static contents


@app.route("/", methods=["GET"])
def onepage():

    response = app.make_response(render_template('onepage.html'))
    response.headers.add('amc-page', 'homepage')
    return response


# URLs for apis


@app.route("/get_amc_version", methods=["GET"])
def get_amc_version():

    try:
        f = open('/opt/amc/amc_version', 'r')
        amc_version = f.read()
        f.close()
    except:
        amc_version = "Error reading AMC version"
    amc_type = "enterprise" if os.path.isfile(ENTERPRISE_FLAG_FILE) else "community"
    return json.dumps(dict(amc_version=amc_version.strip(), amc_type=amc_type))


@app.route("/aerospike/service/clusters/get-cluster-id/<node_address>", methods=["GET"])
def get_cluster_id(node_address):

    response = cluster_manager.get_cluster_info(node_address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_cluster_info(cluster):

    response = cluster_controller.get_info(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/basic", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_cluster_basic_info(cluster):

    response = cluster_controller.get_basic_info(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_node_info(cluster, node_list):

    response = node_controller.get_info(cluster, node_list)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allstats", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_node_stats(cluster, address):

    stats_type = request.args.get("type", "all")
    response = node_controller.get_stats(cluster, address, stats_type)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node_list>",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_info_from_node(cluster, namespace, node_list):

    response = namespace_controller.get_namespace_info_per_node(cluster, str(namespace), node_list)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allstats",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_stats_from_node(cluster, namespace, node):

    stats_type = request.args.get("type", "all")
    response = namespace_controller.get_stats(cluster, str(namespace), node, stats_type)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_info(cluster, namespace_list):

    node_addresses = request.args.get('nodes', None)
    response = namespace_controller.get_namespace_info(cluster, str(namespace_list), node_addresses)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes/<sindex>/nodes/<node>/allstats",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_secondary_index_stats_from_node(cluster, namespace, sindex, node):

    stats_type = request.args.get("type", "all")
    response = namespace_controller.get_index_stats(cluster, str(namespace), str(sindex), node, stats_type)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_secondary_indexes_by_namespace(cluster, namespace):

    response = cluster_controller.get_indexes_by_namespace(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sets",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_sets_by_namespace(cluster, namespace):

    response = cluster_controller.get_sets_by_namespace(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/storage",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_storage_info(cluster, namespace):

    response = namespace_controller.get_storage_info(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_throughput(cluster):

    node_addresses = request.args.get('nodes', None)
    response = cluster_controller.get_throughput(cluster, node_addresses)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput_history", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_throughput_history(cluster):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)
    if start_time is not None:
        start_time = int(start_time)
    if end_time is not None:
        end_time = int(end_time)
    node_addresses = request.args.get('nodes', None)
    response = cluster_controller.get_throughput_history(cluster, node_addresses, start_time, end_time)
    return json.dumps(response)


@app.route("/aerospike/service/clusters", methods=["GET"])
def get_clusters_info():

    response = cluster_controller.get_clusters(cluster_manager)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/jobs/nodes/<node>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_scan_jobs(cluster, node):

    response = udf_controller.get_jobs(cluster, node)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/udfs", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_udfs(cluster):

    response = udf_controller.get_udfs(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_node", methods=["POST"])
@cluster_manager.check_cluster_validity()
def add_node(cluster):

    address = request.form['address']
    response = cluster_controller.add_node(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/remove_node", methods=["POST"])
@cluster_manager.check_cluster_validity()
def remove_node(cluster):

    address = request.form['address']
    response = cluster_controller.remove_node(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespace/<namespace>/add_index", methods=["POST"])
@cluster_manager.check_cluster_validity()
def create_index(cluster, namespace):

    response = crud_controller.create_index(cluster, str(request.form['namespace']), str(request.form['set_name']), str(request.form['index_name']), str(request.form['bin_name']), str(request.form['bin_type']))
    resp = cluster_controller.get_indexes_by_namespace(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespace/<namespace>/drop_index", methods=["POST"])
@cluster_manager.check_cluster_validity()
def drop_index(cluster, namespace):

    response = crud_controller.drop_index(cluster, str(namespace), str(request.form['index_name']))
    resp = cluster_controller.get_indexes_by_namespace(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_udf", methods=["POST"])
@cluster_manager.check_cluster_validity()
def add_udf(cluster):

    response = crud_controller.add_udf(cluster, str(request.form['file_name']), str(request.form['file_contents']), str(request.form['udf_type']))
    resp = udf_controller.get_udfs(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/drop_udf", methods=["POST"])
@cluster_manager.check_cluster_validity()
def drop_udf(cluster):

    response = crud_controller.drop_udf(cluster, str(request.form['file_name']))
    resp = udf_controller.get_udfs(cluster)
    return json.dumps(response)


@app.after_request
def after_request(response):

    response.headers.add('Access-Control-Allow-Origin', '*')
    response.headers.add('Access-Control-Allow-Credentials', 'true')
    response.headers.add('Access-Control-Max-Age', 10)
    response.headers.add('Access-Control-Allow-Methods', 'GET')
    response.headers.add('Cache-Control',
        'must-revalidate, post-check=0, pre-check=0')
    content_type = response.headers.get('Content-Type')
    if content_type.find("text/html") != -1:
        try:
            homepage = response.headers['amc-page']
        except:
            response.headers['Content-Type'] = "application/json"
            response.headers['Cache-Control'] += ', no-cache'
            return response
    expiry_time = datetime.datetime.utcnow() + datetime.timedelta(30)
    response.headers["Expires"] = expiry_time.strftime("%a, %d %b %Y %H:%M:%S GMT")
    return response


app.debug = True

e = eventlet.spawn(background_tasks.perform_scheduled_tasks, cluster_manager, app)
try:
    f = open('/opt/amc/amc_version', 'r')
    amc_version = f.read()
    f.close()
except:
    amc_version = "Error reading AMC version"
sys.stderr.write("AMC Version: %s\n" % amc_version)

if __name__ == "__main__":

    app.run(host='0.0.0.0', port=5001)
